attribute vec3 position;
attribute vec3 normal;
attribute vec2 texcoord;
attribute vec3 tangent;

#if defined(USE_GPU_BONE)
attribute vec4 boneIds;
attribute vec4 weights;
const int MAX_BONES = 112;
uniform mat4 bones[MAX_BONES];
#endif

uniform mat4 worldMat;
uniform mat4 viewMat;
uniform mat4 projectionMat;


uniform float cameraFar;
uniform float cameraNear;

varying vec2 vTexcoord;
varying vec3 vNormal;
varying vec3 vWorldPosition;
varying vec3 vTangent;

float LinearizeDepth(float depth)
{
    float near = cameraNear;
    float far = cameraFar;
    float z = depth * 2.0 - 1.0; // back to NDC
    float dep = (2.0 * near * far) / (far + near - z * (far - near));
    return ((dep - near) / (far - near));
}

void main(){
    //bone
#if defined(USE_GPU_BONE)
    mat4 boneTransform = bones[int(boneIds[0])] * weights[0];
    boneTransform     += bones[int(boneIds[1])] * weights[1];
    boneTransform     += bones[int(boneIds[2])] * weights[2];
    boneTransform     += bones[int(boneIds[3])] * weights[3];

    vec4 bonePos = boneTransform * vec4(position, 1.0);
    vec4 boneNorm = boneTransform * vec4(normal, 0.0);
    vec4 boneTangent = boneTransform * vec4(tangent, 0.0);
#else
    vec4 bonePos = vec4(position, 1.0);
    vec4 boneNorm = vec4(normal, 0.0);
    vec4 boneTangent = vec4(tangent, 0.0);
#endif

    vTexcoord = texcoord;
    vTangent = mat3(worldMat) * vec3(boneTangent);
    vWorldPosition = vec3(worldMat * bonePos);     //transform to then     worldMat
    vNormal = mat3(worldMat) * vec3(boneNorm);
    vec4 unLinerPosition = projectionMat * viewMat * worldMat * bonePos;
    gl_Position = unLinerPosition;
}